@@ -12,14 +12,13 @@ from nautobot.dcim.choices import (
     PortTypeChoices,
 )
 
+from nautobot.core.models import BaseModel
 from nautobot.dcim.constants import REARPORT_POSITIONS_MAX, REARPORT_POSITIONS_MIN
-
 from nautobot.extras.models import CustomFieldModel, ObjectChange, RelationshipModel
 from nautobot.extras.utils import extras_features
-from nautobot.core.models import BaseModel
 from nautobot.utilities.fields import NaturalOrderingField
 from nautobot.utilities.ordering import naturalize_interface
-from nautobot.utilities.utils import serialize_object
+from nautobot.utilities.utils import serialize_object, serialize_object_v2
 from .device_components import (
     ConsolePort,
     ConsoleServerPort,
@@ -31,7 +30,6 @@ from .device_components import (
     RearPort,
 )
 
-
 __all__ = (
     "ConsolePortTemplate",
     "ConsoleServerPortTemplate",
@@ -72,14 +70,22 @@ class ComponentTemplateModel(BaseModel, CustomFieldModel, RelationshipModel):
         except ObjectDoesNotExist:
             # The parent DeviceType has already been deleted
             device_type = None
+
         return ObjectChange(
             changed_object=self,
             object_repr=str(self),
             action=action,
-            related_object=device_type,
             object_data=serialize_object(self),
+            object_data_v2=serialize_object_v2(self),
+            related_object=device_type,
         )
 
+    def instantiate_model(self, model, device, **kwargs):
+        """
+        Helper method to self.instantiate().
+        """
+        return model(device=device, name=self.name, label=self.label, description=self.description, **kwargs)
+
 
 @extras_features(
     "custom_fields",
@@ -98,7 +104,7 @@ class ConsolePortTemplate(ComponentTemplateModel):
         unique_together = ("device_type", "name")
 
     def instantiate(self, device):
-        return ConsolePort(device=device, name=self.name, label=self.label, type=self.type)
+        return self.instantiate_model(model=ConsolePort, device=device, type=self.type)
 
 
 @extras_features(
@@ -118,7 +124,7 @@ class ConsoleServerPortTemplate(ComponentTemplateModel):
         unique_together = ("device_type", "name")
 
     def instantiate(self, device):
-        return ConsoleServerPort(device=device, name=self.name, label=self.label, type=self.type)
+        return self.instantiate_model(model=ConsoleServerPort, device=device, type=self.type)
 
 
 @extras_features(
@@ -150,10 +156,9 @@ class PowerPortTemplate(ComponentTemplateModel):
         unique_together = ("device_type", "name")
 
     def instantiate(self, device):
-        return PowerPort(
+        return self.instantiate_model(
+            model=PowerPort,
             device=device,
-            name=self.name,
-            label=self.label,
             type=self.type,
             maximum_draw=self.maximum_draw,
             allocated_draw=self.allocated_draw,
@@ -210,10 +215,9 @@ class PowerOutletTemplate(ComponentTemplateModel):
             power_port = PowerPort.objects.get(device=device, name=self.power_port.name)
         else:
             power_port = None
-        return PowerOutlet(
+        return self.instantiate_model(
+            model=PowerOutlet,
             device=device,
-            name=self.name,
-            label=self.label,
             type=self.type,
             power_port=power_port,
             feed_leg=self.feed_leg,
@@ -245,10 +249,9 @@ class InterfaceTemplate(ComponentTemplateModel):
         unique_together = ("device_type", "name")
 
     def instantiate(self, device):
-        return Interface(
+        return self.instantiate_model(
+            model=Interface,
             device=device,
-            name=self.name,
-            label=self.label,
             type=self.type,
             mgmt_only=self.mgmt_only,
         )
@@ -307,10 +310,9 @@ class FrontPortTemplate(ComponentTemplateModel):
             rear_port = RearPort.objects.get(device=device, name=self.rear_port.name)
         else:
             rear_port = None
-        return FrontPort(
+        return self.instantiate_model(
+            model=FrontPort,
             device=device,
-            name=self.name,
-            label=self.label,
             type=self.type,
             rear_port=rear_port,
             rear_port_position=self.rear_port_position,
@@ -341,10 +343,9 @@ class RearPortTemplate(ComponentTemplateModel):
         unique_together = ("device_type", "name")
 
     def instantiate(self, device):
-        return RearPort(
+        return self.instantiate_model(
+            model=RearPort,
             device=device,
-            name=self.name,
-            label=self.label,
             type=self.type,
             positions=self.positions,
         )
@@ -365,7 +366,7 @@ class DeviceBayTemplate(ComponentTemplateModel):
         unique_together = ("device_type", "name")
 
     def instantiate(self, device):
-        return DeviceBay(device=device, name=self.name, label=self.label)
+        return self.instantiate_model(model=DeviceBay, device=device)
 
     def clean(self):
         if self.device_type and self.device_type.subdevice_role != SubdeviceRoleChoices.ROLE_PARENT:
