@@ -774,6 +774,8 @@ class ScheduledJob(BaseModel):
 
     def save(self, *args, **kwargs):
         self.queue = self.queue or None
+        # pass pk to worker task in kwargs, celery doesn't provide the full object to the worker
+        self.kwargs["scheduled_job_pk"] = self.pk
         if not self.enabled:
             self.last_run_at = None
         elif not self.last_run_at:
