@@ -17,6 +17,7 @@ from nautobot.extras.filters import (
     ExportTemplateFilterSet,
     GraphQLQueryFilterSet,
     ImageAttachmentFilterSet,
+    JobFilterSet,
     JobLogEntryFilterSet,
     ObjectChangeFilterSet,
     RelationshipAssociationFilterSet,
@@ -28,7 +29,6 @@ from nautobot.extras.filters import (
     TagFilterSet,
     WebhookFilterSet,
 )
-
 from nautobot.extras.models import (
     ConfigContext,
     CustomLink,
@@ -36,6 +36,7 @@ from nautobot.extras.models import (
     GitRepository,
     GraphQLQuery,
     ImageAttachment,
+    Job,
     JobLogEntry,
     JobResult,
     ObjectChange,
@@ -214,6 +215,11 @@ class ConfigContextTestCase(TestCase):
         params = {"tenant": [tenants[0].slug, tenants[1].slug]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
 
+    def test_search(self):
+        value = self.queryset.values_list("pk", flat=True)[0]
+        params = {"q": value}
+        self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
+
 
 class CustomLinkTestCase(TestCase):
     queryset = CustomLink.objects.all()
@@ -512,6 +518,56 @@ class ImageAttachmentTestCase(TestCase):
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
 
 
+class JobFilterSetTestCase(TestCase):
+    queryset = Job.objects.all()
+    filterset = JobFilterSet
+
+    def test_id(self):
+        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+
+    def test_name(self):
+        params = {"name": ["File Upload Success", "File Upload Failure"]}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+
+    def test_grouping(self):
+        params = {"grouping": ["test_file_upload_pass", "test_file_upload_fail"]}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+
+    def test_installed(self):
+        params = {"installed": True}
+        # 31 local jobs and 3 plugin jobs
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 34)
+
+    def test_enabled(self):
+        params = {"enabled": False}
+        # 31 local jobs and 3 plugin jobs
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 34)
+
+    def test_commit_default(self):
+        params = {"commit_default": False}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 0)
+
+    def test_hidden(self):
+        params = {"hidden": True}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+
+    def test_read_only(self):
+        params = {"read_only": True}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+
+    def test_approval_required(self):
+        params = {"approval_required": True}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 0)
+
+    def test_search(self):
+        params = {"q": "file"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+        value = self.queryset.values_list("pk", flat=True)[0]
+        params = {"q": value}
+        self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
+
+
 class JobLogEntryTestCase(TestCase):
     queryset = JobLogEntry.objects.all()
     filterset = JobLogEntryFilterSet
@@ -649,6 +705,11 @@ class ObjectChangeTestCase(TestCase):
         params = {"changed_object_type_id": ContentType.objects.get(app_label="dcim", model="site").pk}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
 
+    def test_search(self):
+        value = self.queryset.values_list("pk", flat=True)[0]
+        params = {"q": value}
+        self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
+
 
 class RelationshipTestCase(TestCase):
     queryset = Relationship.objects.all()
@@ -841,6 +902,11 @@ class SecretTestCase(TestCase):
         params = {"provider": ["environment-variable"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
 
+    def test_search(self):
+        value = self.queryset.values_list("pk", flat=True)[0]
+        params = {"q": value}
+        self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
+
 
 class SecretsGroupTestCase(TestCase):
     queryset = SecretsGroup.objects.all()
@@ -864,6 +930,11 @@ class SecretsGroupTestCase(TestCase):
         params = {"slug": ["group-1", "group-2"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
 
+    def test_search(self):
+        value = self.queryset.values_list("pk", flat=True)[0]
+        params = {"q": value}
+        self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
+
 
 class SecretsGroupAssociationTestCase(TestCase):
     queryset = SecretsGroupAssociation.objects.all()
@@ -990,6 +1061,9 @@ class StatusTestCase(TestCase):
     def test_search(self):
         params = {"q": "active"}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+        value = self.queryset.values_list("pk", flat=True)[0]
+        params = {"q": value}
+        self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
 
 
 class TagTestCase(TestCase):
@@ -998,10 +1072,16 @@ class TagTestCase(TestCase):
 
     @classmethod
     def setUpTestData(cls):
+        cls.tags = (
+            Tag.objects.create(name="Tag 1", slug="tag-1", color="ff0000"),
+            Tag.objects.create(name="Tag 2", slug="tag-2", color="00ff00"),
+            Tag.objects.create(name="Tag 3", slug="tag-3", color="0000ff"),
+        )
+        cls.site_content_type = ContentType.objects.get_for_model(Site)
+        cls.tags[0].content_types.add(cls.site_content_type)
 
-        Tag.objects.create(name="Tag 1", slug="tag-1", color="ff0000")
-        Tag.objects.create(name="Tag 2", slug="tag-2", color="00ff00")
-        Tag.objects.create(name="Tag 3", slug="tag-3", color="0000ff")
+        for tag in cls.tags[1:]:
+            tag.content_types.add(ContentType.objects.get_for_model(Device))
 
     def test_id(self):
         params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
@@ -1019,6 +1099,19 @@ class TagTestCase(TestCase):
         params = {"color": ["ff0000", "00ff00"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
 
+    def test_content_types(self):
+        params = {"content_types": [f"{self.site_content_type.app_label}.{self.site_content_type.model}"]}
+        filtered_data = self.filterset(params, self.queryset).qs
+        self.assertEqual(filtered_data.count(), 1)
+        self.assertEqual(filtered_data[0], self.tags[0])
+
+    def test_search(self):
+        params = {"q": "tag-1"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+        value = self.queryset.values_list("pk", flat=True)[0]
+        params = {"q": value}
+        self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
+
 
 class WebhookTestCase(TestCase):
     queryset = Webhook.objects.all()
@@ -1077,3 +1170,6 @@ class WebhookTestCase(TestCase):
     def test_search(self):
         params = {"q": "webhook"}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+        value = self.queryset.values_list("pk", flat=True)[0]
+        params = {"q": value}
+        self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
