@@ -1025,7 +1025,6 @@ class JobView(ObjectPermissionRequiredMixin, View):
                     user=request.user,
                     approval_required=job_model.approval_required,
                 )
-                scheduled_job.kwargs["scheduled_job_pk"] = scheduled_job.pk
                 scheduled_job.save()
 
                 if job_model.approval_required:
@@ -1212,7 +1211,7 @@ class ScheduledJobListView(generic.ObjectListView):
     table = tables.ScheduledJobTable
     filterset = filters.ScheduledJobFilterSet
     filterset_form = forms.ScheduledJobFilterForm
-    action_buttons = ("delete",)
+    action_buttons = ()
 
 
 class ScheduledJobBulkDeleteView(generic.BulkDeleteView):
@@ -1581,10 +1580,7 @@ class SecretsGroupListView(generic.ObjectListView):
     filterset = filters.SecretsGroupFilterSet
     filterset_form = forms.SecretsGroupFilterForm
     table = tables.SecretsGroupTable
-    action_buttons = (
-        "add",
-        "delete",
-    )
+    action_buttons = ("add",)
 
 
 class SecretsGroupView(generic.ObjectView):
